package thoughtbot.expandableadapter.models;

import android.os.Parcel;
import android.os.Parcelable;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;

import awais.instagrabber.models.FollowModel;

public class ExpandableGroup implements Parcelable {
    public static final Creator<ExpandableGroup> CREATOR = new Creator<ExpandableGroup>() {
        @NonNull
        @Override
        public ExpandableGroup createFromParcel(@NonNull final Parcel in) {
            return new ExpandableGroup(in);
        }

        @NonNull
        @Override
        public ExpandableGroup[] newArray(final int size) {
            return new ExpandableGroup[size];
        }
    };
    private final String title;
    private final List<FollowModel> items;

    public ExpandableGroup(final String title, final List<FollowModel> items) {
        this.title = title;
        this.items = items;
    }

    protected ExpandableGroup(@NonNull final Parcel in) {
        title = in.readString();
        final byte hasItems = in.readByte();
        final int size = in.readInt();
        if (hasItems != 0x01) items = null;
        else {
            items = new ArrayList<>(size);
            Class<?> type = (Class<?>) in.readSerializable();
            if (type != null) in.readList(items, type.getClassLoader());
        }
    }

    public String getTitle() {
        return title;
    }

    public List<FollowModel> getItems() {
        return items;
    }

    public int getItemCount() {
        return items == null ? 0 : items.size();
    }

    @Override
    public int describeContents() {
        return 0;
    }

    @Override
    public void writeToParcel(@NonNull final Parcel dest, final int flags) {
        dest.writeString(title);
        if (items == null) {
            dest.writeByte((byte) 0x00);
            dest.writeInt(0);
        } else {
            dest.writeByte((byte) 0x01);
            dest.writeInt(items.size());
            final Class<?> objectsType = items.get(0).getClass();
            dest.writeSerializable(objectsType);
            dest.writeList(items);
        }
    }
}